\ ----------------------------------------------------------------------
\ LANGUAGE    : noForth vsn 2.00
\ PROJECT     : noForth, een publiek domein Forth voor de MSP430
\ DESCRIPTION : Remote control with nRF24L01+
\ CATEGORY    : Driver 24L01 + demo, afmeting: 1552 bytes.
\ AUTHOR      : Willem Ouwerkerk, June 09, 2014
\ PREV CHANGE : Jan van Kleef, Sept. 20, 2014
\ LAST CHANGE : Willem Ouwerkerk, Okt. 05, 2014
\ ----------------------------------------------------------------------

\ Voor demo ADC en potmeter toevoegen
\ Op andere toongeneator of leds
\ Knop om van functie te switchen op afstand
\ Niets is teller demo, knop en potmeter is toon demo.

( Launchpad documentatie van software SPI
  
P2 bevat de interface naar de nRF24L01+
P2 0  - CE                      \ Device enable high  x1=Enable
P2 1  - CSN                     \ SPI enable low      x1=Select
P2 2  - CLOCKPULSE              \ Klok                x1=Clock
P2 3  - DATA-OUT                \ Data bitstroom uit  x1=Mosi
P2 4  - DATA-IN                 \ Data bitstroom in   x0=Miso

P1.0  - Led
P1.6  - Led
P1 3  - SW1

Register 08 bevat de lost packets en recent packets in twee nibbles werkt ook!
Een byte fifo werkt prima. Er zit een commando CHECK bij die helpt te controleren
of het rustig of druk is op het gekozen RF kanaal.

Nog te testen: Hogere bitrate, 1 pipe, minder retry's en
een proef met een enkele device adres voor twee modules.
Daarna een proef met drie modules in ster of in een ring!

Lezen en schrijven andere pipes toevoegen, dit zijn n byte adressen
die samen met de eerste 4 bytes van P1 een vijf bytes adres vormen )

: GREEN-ON  40 21 *bis ;    : GREEN-OFF 40 21 *bic ;
: RED-ON    01 21 *bis ;    : RED-OFF   01 21 *bic ;
: LEDS-ON   41 21 *bis ;    : LEDS-OFF  41 21 *bic ;

                    ( Software SPI interface )

: SPI-SETUP         ( -- )
    EF 2A c!  10 2F *bis  10 29 *bis    \ P2.0 to P2.4 for SPI
    1 29 *bic       \ ce  = 0
    2 29 *bis       \ csn = 1
    4 29 *bic       \ clk = 0
    ;

: IO-SETUP          ( -- )
    41 22 *bis      \ P1.1 & P1.6 output
\   41 21 *bic      \ Both outputs off
    08 22 *bic      \ P1.3 input
    08 27 *bis      \ With pullup
    ;

: CLOCK        ( -- )      4 29 *bis  4 29 *bic ;       \ SPI clock
: ACTIVATE     ( -- )      2 29 *bic ;                  \ SPI on
: DEACTIVATE   ( -- )      2 29 *bis ;                  \ SPI off
: TRANSMIT     ( -- )      1 29 *bis  noop 1 29 *bic ;  \ Transmit pulse on CE

\ Schrijf een bit naar SPI-bus, lees een bit van SPI-bus
: WRITE-BIT    ( flag -- ) 80 and if 8 29 *bis else 8 29 *bic then ;
: READ-BIT     ( -- flag ) 10 28 bit* 0<>  clock ;
 
: SPI-OUT           ( x -- )            \ Write x to SPI-bus
    8 0 do   dup write-bit  clock  2*  loop  drop 
    ;

: SPI-IN            ( -- x )            \ Read x from SPI-bus
    0  8 0 do  2*  read-bit 1 and or  loop
    ;

: SPI-I/O           ( x1 -- x2 )        \ Read and write at SPI-bus
    0  8 0 do
        >r  dup write-bit  2*  
        r>  2* read-bit 1 and or
    loop
    nip
    ;

                ( Lezen en schrijven naar en van de nRF24L01 )

: RMASK    ( x1 -- x2 )    1F and ;
: WMASK    ( x1 -- x2 )    1F and  20 or ;

\ De eerste gestuurde byte geeft altijd de interne status terug
\ Die wordt bewaard in de value STATUS bij SPI-COMMAND
VALUE STATUS
: GET-STATUS    ( -- s )    activate  FF spi-i/o  deactivate ;
: SPI-COMMAND   ( c -- )    activate  spi-i/o to status ;

\ Registers in de 24L01 lezen en schrijven
: READ-REG      ( r -- d )  rmask spi-command  spi-in  deactivate ;
: WRITE-REG     ( d r -- )  wmask spi-command  spi-out deactivate ;

\ Lees en schrijf de communicatie adressen, van pipe-0 default: E7E7E7E7E7
: READ-ADDR     ( trxa -- ) rmask spi-command  5 0 do spi-in  loop  deactivate ;
: WRITE-ADDR    ( trxa -- ) wmask spi-command  5 0 do spi-out loop  deactivate ;
: .PIPE-ADDR    ( r -- )    read-addr 5 0 do  u.  loop ; ( o.a. 0A, 0B, 10 )

                ( nRF24L01 besturings commando's en setup )

\ Empty RX or TX data pipe
: FLUSH-RX      ( -- )      E2 spi-command deactivate ;
: FLUSH-TX      ( -- )      E1 spi-command deactivate ;

: RESET         ( -- )      70 07 write-reg ;

: SETUP24L01    ( -- )
    spi-setup  -1 to status  io-setup
    A0 ms
    3F 01 write-reg       \ Auto Ack all pipes
    03 02 write-reg       \ Pipe-0 & Pipe 1 on
    03 03 write-reg       \ Five byte address
    FF 04 write-reg       \ 15 retry's 4000 us
    4C 05 write-reg       \ channel 76
    07 06 write-reg       \ 1 Mbps, max. power
    01 11 write-reg       \ 1 bytes payload in P0
    01 12 write-reg       \ 1 bytes payload in P1
    flush-tx
    flush-rx
    0C 00 write-reg       \ Enable CRC, 2 bytes, power down?
    reset                 \ Reset flags
    ;

\ Elementaire commando set voor de 24L01
: READ-RX       ( -- )      61 spi-command  spi-in deactivate ; \ Receive
: WRITE-TXA     ( c -- )    A0 spi-command spi-out deactivate ; \ Send with Ack check
: .LOST         ( -- )      08 read-reg 0F and 2 .r space ;     \ Show lost packages

\ Diverse testen op de status van de ontvanger en zender
: RX?           ( -- f )    07 read-reg 40 and 0<> ;    \ Data received
: TX?           ( -- f )    07 read-reg 20 and 0<> ;    \ Ack received
: FIFO-EMPTY?   ( -- f )    17 read-reg 01 and 0<> ;    \ Pipeline empty
: FIFO-FULL?    ( -- f )    17 read-reg 01 and 0= ;     \ Pipeline full
: MAXRETRY?     ( -- f )    07 read-reg 10 and 0<> ;    \ Transmit failed

                    ( Zend en ontvang commando's voor nRF24L01 )

: WRITE-MODE    ( -- )      
    1 29 *bic                           \ CE low, receive mode off
    0E 00 write-reg                     \ Power up module as transmitter
    reset
    ;

: READ-MODE     ( -- )      
    0F 00 write-reg                     \ Power up module as receiver
    1 29 *bis                           \ Enable receive mode
    ;

: XEMIT         ( c -- )
    flush-tx                            \ Empty pipeline
    write-txa  1 ms  transmit           \ Send char
    80 0 do  tx? if  leave  then  loop  \ Wait for Ack
    tx? 0= maxretry? and if ." fout " then \ No Ack and retry overflow!
    reset                               \ Reset flags
    ;

: XKEY          ( -- c )
    begin  rx? until                    \ Wait for char
    read-rx                             \ Read char
    reset  2 ms  flush-rx               \ Empty pipeline
    ;  

                            ( nRF24L01 demo )

 ( Send n1 the receiver, response from the receiver is n2 )
: SEND          ( n1 -- n2 )
    green-on                            \ Led on  
    write-mode                          \ Power up module as transmitter
    cr  xemit   .lost                   \ Send data, show lost packages
    read-mode                           \ Power up as receiver, reset flags
    xkey  dup .                         \ Read and show response
    1 29 *bic                           \ CE low, receive mode off
    green-off                           \ Led off 
    80 ms                               \ Wait
    ;

 ( Receive data from transmitter, add 1 to it and send back ) 
: RECEIVE         ( -- )       
    cr read-mode                        \ Power up module as receiver
    xkey                                \ Read
    red-on                              \ Led on      
    write-mode                          \ Power up module as transmitter
    dup 1+ xemit                        \ Make copy, increase and send as responce
    .lost  .                            \ Show lost packages and data
    red-off                             \ Led off  
    ;

\ Test of er andere 2.4GHz zenders actief zijn op ingestelde frequentie
: CHECK         ( -- )
    setup24L01
    F0 F0 F0 F0 E1 0A write-addr        \ Receive address P0
    F0 F0 F0 F0 D2 0B write-addr        \ Receive address P1
    F0 F0 F0 F0 E1 10 write-addr        \ Transmit address
    get-status .                        \ Print status
    begin 
        read-mode  1 ms                 \ Receive on, wait
        09 read-reg 1 .R  write-mode    \ Test for carrier on used frequency
    key? until
    ;

: XWAIT         ( -- )                  \ Wait for RF data, printing dots
    read-mode  begin  10 ms ch . emit  rx? until
    ;

: SPI-SEND      ( -- )
    leds-on
    setup24L01
    F0 F0 F0 F0 E1 0A write-addr        \ Receive address P0
    F0 F0 F0 F0 D2 0B write-addr        \ Receive address P1
    F0 F0 F0 F0 E1 10 write-addr        \ Transmit address
    get-status .                        \ Print status
    125 ms  leds-off
    0  begin  send  key? until  drop
    leds-off
    ;

: SPI-REC       ( -- )
    leds-on
    setup24L01
    F0 F0 F0 F0 D2 0A write-addr        \ Receive address P0
    F0 F0 F0 F0 E1 0B write-addr        \ Receive address P1
    F0 F0 F0 F0 D2 10 write-addr        \ Transmit address
    get-status .                        \ Print status
    125 ms  leds-off
    begin  xwait  receive  key? until
    leds-off
    ;

shield 24L01\
freeze

                              ( End )
