\ ----------------------------------------------------------------------
\ LANGUAGE    : noForth vsn 2.00
\ PROJECT     : noForth, een publiek domein Forth voor de MSP430
\ DESCRIPTION : Afstand besturing demo met de nRF24L01+
\ CATEGORY    : Demo voor nRF24L01, afmeting: 690 bytes.
\ AUTHOR      : Willem Ouwerkerk, Okt. 06, 2014
\ ----------------------------------------------------------------------
\ Deze afstandbediening maakt gebruik van het zenden en ontvangen in
\ beide richtingen om een terugmelding van de geselecteerde functie te 
\ geven. En is een voorbeeld van data lezen op afstand.
\ ----------------------------------------------------------------------

\ +n is 0 or 10
: MSPKEY            ( -- f )            \ True if S2 key on MSP pressed?
    s? 0=
    ;

: .MODE             ( f -- )
    ." Mode is "  if  ." ADC "  else  ." COUNTER "  then  cr
    ;

: REMOTE            ( -- )
    setup24L01
    F0 F0 F0 F0 E1 0A write-addr        \ Receive address P0
    F0 F0 F0 F0 D2 0B write-addr        \ Receive address P1
    F0 F0 F0 F0 E1 10 write-addr        \ Transmit address
    get-status .                        \ Print status
    cr ." Transmitter " 
    begin
        mspkey if                       \ MSP button pressed?
            cr  leds-on
            write-mode ch T xemit .lost \ Send function toggle command
            read-mode  xkey  .mode      \ Get and show function mode
            100 ms  leds-off
            begin
                read-mode               \ Power up as receiver, reset flags
                xkey  .                 \ Read and show response
                1 29 *bic               \ CE low, receive mode off
            mspkey key? or until        \ Button pressed?
       then
    key? until
    ;

04A constant ADC-ENABLE                 \ Used ADC control registers
1B0 constant ADC-CTRL0
1B2 constant ADC-CTRL1
1B4 constant ADC-MEM
1BC constant ADC-START

\ ADC on and sample time at 64 clocks
: ADC-SETUP ( -- )
    02 adc-ctrl0 **bic                  ( Clear ENC )
    80 adc-enable c!                    ( P1.7 = ADC in )
    1810 adc-ctrl0 !                    ( Sampletime 64 clocks, ADC on )
    ;

\ We need to clear the ENC bit before setting a new input channel
: ADC       ( +n -- u )
    02 adc-ctrl0 **bic                  ( Clear ENC )
    F000 and 80 or adc-ctrl1 !          ( Select input, MCLK/5 )
    03 adc-ctrl0 **bis                  ( Set ENC & ADC10SC )
    begin 1 adc-ctrl1 bit** 0= until    ( ADC10 busy? )
    adc-mem @                           ( Read result )
    ;

: POTMETER      7000 adc 2 rshift ;     \ Level at P1.7 as 8-bit value

value T?                                \ Operating mode memory
value NR                                \ Demo counter memory

: TRX?              ( -- 0|-1 )         \ Wait 50 ms for data
    read-mode  0  50 0 do
        1 ms  rx? if  1-  leave  then
    loop
    ;

: SEND-ADC          ( -- )
    cr ." ADC mode " 
    begin
        write-mode  potmeter xemit .lost
    trx? key? or until
    ;

: SEND-COUNTER      ( -- )
    cr ." Counter mode "
    begin
        write-mode  nr xemit .lost  1 +to nr
    trx? key? or until
    ;

: SEND-DATA         ( -- )
    t? if  send-adc exit  then  send-counter
    ;
    
: RECEIVER          ( -- )
    setup24L01
    F0 F0 F0 F0 D2 0A write-addr        \ Receive address P0
    F0 F0 F0 F0 E1 0B write-addr        \ Receive address P1
    F0 F0 F0 F0 D2 10 write-addr        \ Transmit address
    get-status .                        \ Print status
    adc-setup  cr ." Receiver "         \ Set HW ready
    xwait  cr                           \ Wait for first RF data
    begin
        xkey  ch T = if                 \ Toggle command
            t? 0= to t?                 \ Yes, invert flag
        then
        write-mode  t? xemit  .lost     \ Sent flag back
        100 ms  send-data               \ Sent data back
    key? until
    ;

shield remote\
freeze

                            ( End )
